/*
misc.c   Support routines
*/

#include "odmr.h"

volatile int sigalarm;

void trap_alarm(int signum){
  sigalarm = 1;
  if (debug) fprintf(stderr,"*Timeout*\n");
}

void trap_hup(int signum){
  if (debug) fprintf(stderr,"*Hangup*\n");
  signal(SIGHUP,trap_hup);
}

int writen(int fd,char *ptr, int nbytes){
  int nleft,nwritten;
  nleft = nbytes;
  while (nleft >0) {
    nwritten = write(fd,ptr,nleft);
    if (sigalarm) return -1;
    if (nwritten <=0 ) return(nwritten); /* error */
    nleft -= nwritten;
    ptr += nwritten;
  }
  return(nbytes - nleft);
}

int sendstr(int fd, char *str){
  int res,n = strlen(str);
  sigalarm = 0;
  signal(SIGALRM,trap_alarm);
  alarm(TIMEOUT);
  if (debug) fprintf(stderr,"> %s",str);
  res = writen(fd,str,n);
  alarm(0);
  signal(SIGALRM,SIG_IGN);
  if ((res != n) || sigalarm) 
    return (-1);
  else
    return(0);
}

int get_response(int fd){
  char *p,*pp,*ch;
  struct timeval tv;
  fd_set select_set;
  int n,res,size;
  p = buff; size = MAX_STR-1;
  buff[0] = 0;
  for (;;){
    FD_ZERO(&select_set);
    FD_SET(fd,&select_set);
    tv.tv_sec = TIMEOUT;
    tv.tv_usec = 0;
    if ( (res = select(fd+1,&select_set,NULL,NULL,&tv)) <=0) return -1;
    if (FD_ISSET(fd,&select_set)){
      if ( (res = recv(fd,p,size,0)) <= 0) return -1;
      p[res] = 0;
      if (debug) fprintf(stderr,"< %s",p);
      size -= res;
      p += res;
      /* Try to see if we have a complete response */
      pp=buff;
      for (ch=buff;ch<p-1;ch++)
	if (*ch=='\n') pp = ch+1;
      if ((pp[3] !='-') || (size<10)) break;
    }
  }
  if (sscanf(buff,"%3d",&n) == 0) return(-1); /* No numeric response from host */
  if (debug) fprintf(stderr,"Response: %d\n",n);
  return (n);
}

int send_cmd(int fd, char * cmd){
  int n=sendstr(fd,cmd);
  if (n<0)
    return (n);
  else
    return (get_response(fd));
}


/* Spawn an MTA and pass it the authenticated pipe */
void spawn_mail_handler(int serverfd){
  pid_t pid;
  syslog(LOG_INFO,"Transferring mail via %s %s",MTA,MTA_PARAM);
  if ( (pid = fork()) < 0 ) syslog(LOG_ERR,"Fork 1 failed\n");
  else if (pid == 0) { /* First child */
    if ( (pid = fork()) < 0 ) syslog(LOG_ERR,"Fork 2 failed\n");
    else if (pid > 0)
      exit(0); /* parent from second fork == first child */
    /* Second child. Parent is init because first child has exited above */
    /* Set input,output and error to the pipe */
    dup2(serverfd,0);
    dup2(serverfd,1);
    dup2(serverfd,2);
    execl(MTA,MTA,MTA_PARAM);
    exit(1);
  }
  if (waitpid(pid,NULL,0) != pid) /* Reap first child */
    syslog(LOG_WARNING,"Waitpid failed");
}

void daemon_init(void){
  pid_t pid;
  if ( (pid = fork()) < 0) return; /* failed */
  else if (pid != 0) exit(0); /* Lose the parent */
  /* Child continues */
  setsid(); /* Start our own session leader */
  chdir("/");
  umask(0);
}
