/*
  odmr.c             Simple On-Demand Mail Relay client
  Copyright (c) 2000 David Godfrey (dave@deltass.co.uk)
*/

#include "odmr.h"

char buff[MAX_STR];

int main (int argc, char *argv[]){
  int serverfd=0,response;
  struct sockaddr_in serv_addr;
  int count=0,state=10,i;
  char challenge[MAX_STR],*p;
  unsigned char digest[16];
  if (!debug) daemon_init();
  signal(SIGHUP,trap_hup);
  signal(SIGPIPE,SIG_IGN); /* Handle pipe errors ourself */
  openlog("mailkeep",(LOG_PID /*| LOG_PERROR*/),LOG_MAIL);

  for (;;){
    switch (state){
      case 10:
	/* fill serv_addr with ODMR server details */
	bzero((char *) &serv_addr,sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = inet_addr(SERVER_ADDR);
	serv_addr.sin_port = htons(SERVER_PORT);
        if ( (serverfd = socket(AF_INET,SOCK_STREAM,0)) < 0){
	  syslog(LOG_ERR,"Can't open stream socket");
	  state = 60;
	  break;
	}
	/* Connect to server */
        if (connect(serverfd,(struct sockaddr *) &serv_addr,sizeof(serv_addr))<0){
	  syslog(LOG_WARNING,"Can't connect to server");
          state = 60;
	  break;
	}
	state = 20;
	break;
      case 20:    /* Greetings */
	if (get_response(serverfd) / 100 != 2) {
	  syslog(LOG_ERR,"Negative response in banner greeting: %s",buff);
	  state = 60;
	  break;
	}
	if (send_cmd(serverfd,EHLO_DOMAIN) / 100 != 2){
	  syslog(LOG_ERR,"Invalid response to EHLO: %s",buff);
	  state = 60;
	  break;
	}
	state = 30;
	break;
      case 30:    /* Authenticate */
	if (send_cmd(serverfd,"AUTH CRAM-MD5\r\n") / 100 != 3){
	  syslog(LOG_ERR,"Invalid response to AUTH: %s",buff);
	  state = 50;
	  break;
	}
	buff[strlen(buff)-2] = 0; /* strip cr/lf */
	if (auth_b64decode(buff+4,challenge) <0){
	  syslog(LOG_ERR,"Bad base 64 string in challenge");
	  state = 50;
	  break;
	}
	compute_cram_md5(SECRET,challenge,digest);
	/* Construct the response in `challenge' */
	sprintf(challenge,"%s",SERVER_ACCOUNT);
	p = challenge;
	while (*p) p++;
	*p++ = ' ';
	for (i=0;i<16;i++) {
	  sprintf(p,"%02x",digest[i]);
	  p += 2;
	}
  if (debug) fprintf(stderr,"Response: %s\n",challenge);
	auth_b64encode(challenge,strlen(challenge),buff);
	strcat(buff,"\r\n");
	if (send_cmd(serverfd,buff) / 100 != 2){
	  syslog(LOG_ERR,"Authentication failed: %s",buff);
	  state = 50;
	  break;
	}
	state = 40;
	break;
      case 40:
	/* Authenticated Turn */
	response=send_cmd(serverfd,"ATRN\r\n");
	if ( (response==451) && (count++<5)){
	  syslog(LOG_WARNING,"Server too busy? %s",buff);
	  sleep(60); /* Unable to process ATRN request now */
	  break;
	}
	count=0;
	if (response==453)
	  syslog(LOG_INFO,"No mail waiting");
	else if (response / 100 == 2){
	  spawn_mail_handler(serverfd);
	  state=60;
	  break;
	}
	else
	  syslog(LOG_ERR,"Invalid response to ATRN: %s",buff);
	state = 50;
	break;
      case 50:    /* quit */
	if (send_cmd(serverfd,"QUIT\r\n") / 100 != 2)
	     syslog(LOG_WARNING,"Rude response to QUIT: %s",buff);
	state = 60;
	break;
      case 60:
	close(serverfd);
	sleep(SLEEP_TIME);
	state = 10;
	break;
    }	
  }
}
